@@ -13,8 +13,8 @@ typedef enum AlertDescription {
 	GNUTLS_A_DECRYPT_ERROR, GNUTLS_A_EXPORT_RESTRICTION=60, 
 	GNUTLS_A_PROTOCOL_VERSION=70, GNUTLS_A_INSUFFICIENT_SECURITY, 
 	GNUTLS_A_INTERNAL_ERROR=80, GNUTLS_A_USER_CANCELED=90,
-	GNUTLS_A_NO_RENEGOTIATION=100,
-	GNUTLS_A_CERTIFICATE_UNOBTAINABLE=111
+	GNUTLS_A_NO_RENEGOTIATION=100, GNUTLS_A_UNSUPPORTED_EXTENSION=110,
+	GNUTLS_A_CERTIFICATE_UNOBTAINABLE=111, GNUTLS_A_UNRECOGNIZED_NAME=112
 } gnutls_alert_description;
 
 gnutls_alert_description gnutls_alert_get( gnutls_session session);
