@@ -2,11 +2,11 @@
 # define GNUTLS_PK_H
 
 int _gnutls_pkcs1_rsa_encrypt(gnutls_datum * ciphertext, gnutls_datum plaintext,
-		      MPI * params, int params_len, int btype);
+		      MPI * params, uint params_len, uint btype);
 int _gnutls_dsa_sign(gnutls_datum * signature, const gnutls_datum *plaintext,
-		      MPI *params, int params_len);
+		      MPI *params, uint params_len);
 int _gnutls_pkcs1_rsa_decrypt(gnutls_datum * plaintext, gnutls_datum ciphertext,
-		      MPI * params, int params_len, int btype);
+		      MPI * params, uint params_len, uint btype);
 int _gnutls_rsa_verify( const gnutls_datum* vdata, const gnutls_datum *ciphertext, 
 	MPI* params, int params_len, int btype);
 int _gnutls_dsa_verify( const gnutls_datum* vdata, const gnutls_datum *sig_value, 
